import { BehaviorSubject } from 'rxjs';
import { zoomIdentity, select, zoom, event } from 'd3';
import { Injectable, Component, Input, NgModule, defineInjectable, inject } from '@angular/core';
import { HttpClient, HttpClientModule } from '@angular/common/http';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class PrimitiveElement {
    constructor() { }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class PrimitiveRect extends PrimitiveElement {
    constructor() {
        super();
        this.rx = 2,
            this.ry = 2,
            this.type = 'rect';
        this.width = 0;
        this.height = 0;
        this.position = { x: 0, y: 0 };
        this.cssClass = 'bpmn-flows-rect';
        this.textCssClass = 'bpmn-flows-text';
        this.iconCssClass = 'bpmn-flows-task-icon';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class UserTaskRect extends PrimitiveRect {
    constructor() {
        super();
        this.icon = 'fa fa-user';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class PrimitiveCircle extends PrimitiveElement {
    constructor() {
        super();
        this.type = 'circle';
        this.ratio = null;
        this.position = { x: 0, y: 0 };
        this.cssClass = 'bpmn-flows-circle';
        this.textCssClass = 'bpmn-flows-circle-name';
    }
    /**
     * @param {?} ratio
     * @return {?}
     */
    setRatio(ratio) {
        this.width = ratio;
        this.height = ratio;
        this.ratio = ratio;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class StartEventCircle extends PrimitiveCircle {
    constructor() {
        super();
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class EndEventCircle extends PrimitiveCircle {
    constructor() {
        super();
        this.cssClass += ' bpmn-flows-end-event-circle';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class MultiInstanceRect extends PrimitiveRect {
    constructor() {
        super();
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class PrimitiveSequence extends PrimitiveElement {
    constructor() {
        super();
        this.waypoints = [];
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class SequenceFlowPath extends PrimitiveSequence {
    constructor() {
        super();
        this.cssClass = 'bpmn-flows-path';
        this.textCssClass = 'bpmn-flows-path-name';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class PrimitiveRhombus extends PrimitiveElement {
    constructor() {
        super();
        this.rx = 2,
            this.ry = 2,
            this.type = 'polygon';
        this.width = 0;
        this.height = 0;
        this.position = { x: 0, y: 0 };
        this.points = '';
        this.textCssClass = 'bpmn-flows-text';
        this.cssClass = 'bpmn-flows-gateway';
        this.iconCssClass = 'bpmn-flows-gateway-icon';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class ExclusiveGatewayRhombus extends PrimitiveRhombus {
    constructor() {
        super();
        this.icon = 'fa fa-times';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class ParallelGatewayRhombus extends PrimitiveRhombus {
    constructor() {
        super();
        this.icon = 'fa fa-plus';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class ManualTaskRect extends PrimitiveRect {
    constructor() {
        super();
        this.icon = 'fa fa-handshake';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class ScriptTaskRect extends PrimitiveRect {
    constructor() {
        super();
        this.icon = 'fa fa-code';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class LaneRect extends PrimitiveRect {
    constructor() {
        super();
        this.childLanes = [];
        this.flowRefs = [];
        this.textCssClass = 'bpmn-flows-lane-name';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class ParticipantRect extends PrimitiveRect {
    constructor() {
        super();
        this.lanes = [];
        this.textCssClass = 'bpmn-flows-lane-name';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
/** @type {?} */
const BpmnTypes = {
    'userTask': UserTaskRect,
    'scriptTask': ScriptTaskRect,
    'manualTask': ManualTaskRect,
    'startEvent': StartEventCircle,
    'endEvent': EndEventCircle,
    'subProcess': MultiInstanceRect,
    'sequenceFlow': SequenceFlowPath,
    'exclusiveGateway': ExclusiveGatewayRhombus,
    'parallelGateway': ParallelGatewayRhombus,
    'lane': LaneRect,
    'participant': ParticipantRect
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class Shape {
    /**
     * @param {?} xmlShape
     */
    constructor(xmlShape) {
        this.childrens = [];
        this.form = xmlShape.form;
        this.id = xmlShape.id;
        this.name = xmlShape.name;
        this.in = [];
        this.out = [];
        this.element = new BpmnTypes[xmlShape.type]();
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class SequenceFlow {
    /**
     * @param {?} id
     * @param {?} name
     * @param {?} source
     * @param {?} target
     */
    constructor(id, name, source, target) {
        this.id = id;
        this.name = name;
        this.source = source;
        this.target = target;
        this.element = new SequenceFlowPath();
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class BpmnElements {
    constructor() {
        this.shapes = {};
        this.sequences = {};
    }
    /**
     * @return {?}
     */
    bindElementAndSequence() {
        /** @type {?} */
        const sequenceKeys = Object.keys(this.sequences);
        for (const sequenceKey of sequenceKeys) {
            /** @type {?} */
            const sequence = this.sequences[sequenceKey];
            sequence.source.out.push(sequence);
            sequence.target.in.push(sequence);
        }
    }
    /**
     * @param {?} size
     * @return {?}
     */
    calculateRhombusPoints(size) {
        return size / 2 + ',0 ' + size + ',' + size / 2 + ' ' + size / 2 + ',' + size + ' 0,' + size / 2;
    }
    /**
     * @param {?} xmlElement
     * @return {?}
     */
    createElement(xmlElement) {
        /** @type {?} */
        const shape = new Shape(xmlElement);
        this.shapes[shape.id] = shape;
        return shape;
    }
    /**
     * @param {?} id
     * @param {?} name
     * @param {?} source
     * @param {?} target
     * @return {?}
     */
    createSequence(id, name, source, target) {
        /** @type {?} */
        const sourceShape = this.shapes[source];
        /** @type {?} */
        const targetShape = this.shapes[target];
        if (!sourceShape || !targetShape) {
            console.warn(source);
            console.warn(target);
            throw new Error('Sequencia incompleta.');
        }
        /** @type {?} */
        const sequence = new SequenceFlow(id, name, sourceShape, targetShape);
        this.sequences[id] = sequence;
    }
    /**
     * @param {?} id
     * @param {?} position
     * @return {?}
     */
    setSequenceNamePlacement(id, position) {
        /** @type {?} */
        const sequence = this.sequences[id];
        sequence.element.textPosition = position;
    }
    /**
     * @param {?} id
     * @param {?} waypoints
     * @return {?}
     */
    setSequenceWaypoints(id, waypoints) {
        /** @type {?} */
        const sequence = this.sequences[id];
        if (!sequence) {
            throw new Error('Sequência não criada: ' + id);
        }
        /** @type {?} */
        const element = sequence.element;
        element.waypoints.push(waypoints);
    }
    /**
     * @param {?} id
     * @param {?} width
     * @param {?} height
     * @param {?} position
     * @return {?}
     */
    setShapeAttributes(id, width, height, position) {
        /** @type {?} */
        const shape = this.shapes[id];
        if (!shape) {
            console.warn('Shape não existe: ' + id);
            throw new Error('Shape não conhecida.');
        }
        if (shape.element instanceof PrimitiveRect) {
            /** @type {?} */
            const element = shape.element;
            element.width = width;
            element.height = height;
            element.position = position;
        }
        else if (shape.element instanceof PrimitiveCircle) {
            /** @type {?} */
            const element = shape.element;
            /** @type {?} */
            let ratio = ((width + height) / 4);
            element.setRatio(ratio);
            element.position = position;
        }
        else if (shape.element instanceof PrimitiveRhombus) {
            /** @type {?} */
            const element = shape.element;
            element.width = width;
            element.height = height;
            element.position = position;
            element.points = this.calculateRhombusPoints(element.width);
        }
        else {
            console.warn(shape.element);
            throw new Error('Shape não conhecida.');
        }
    }
    /**
     * @param {?} id
     * @param {?} position
     * @return {?}
     */
    setShapeNamePlacement(id, position) {
        /** @type {?} */
        let shape = this.shapes[id];
        shape.element.textPosition = position;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class BpmnImporter {
    /**
     * @param {?} xml
     */
    constructor(xml) {
        this.bpmnElements = new BpmnElements();
        this.bpmnElements$ = new BehaviorSubject(null);
        this.getElements(xml);
    }
    /**
     * @param {?} dom
     * @param {?} type
     * @param {?} father
     * @return {?}
     */
    createLane(dom, type, father) {
        /** @type {?} */
        const childs = this.htmlCollectionToArray(dom.children);
        if (type === 'laneSet') {
            for (let lane of childs) {
                this.importElement(lane, null);
            }
            return;
        }
        /** @type {?} */
        const xmlElement = {
            id: dom.getAttribute('id'),
            name: dom.getAttribute('name'),
            type: type
        };
        /** @type {?} */
        const shape = this.bpmnElements.createElement(xmlElement);
        /** @type {?} */
        const element = /** @type {?} */ (shape.element);
        if (father) {
            shape.father = father;
        }
        for (let child of childs) {
            if (child.tagName === 'bpmn:flowNodeRef') {
                element.flowRefs.push(child.childNodes[0]);
            }
            else if (child.tagName === 'bpmn:childLaneSet') {
                /** @type {?} */
                const laneChilds = this.htmlCollectionToArray(child.children);
                for (let laneChild of laneChilds) {
                    shape.childrens.push(this.importElement(laneChild, shape));
                }
            }
        }
        return shape;
    }
    /**
     * @param {?} dom
     * @param {?} type
     * @param {?} father
     * @return {?}
     */
    createShape(dom, type, father) {
        try {
            /** @type {?} */
            const xmlElement = {
                form: dom.getAttribute('camunda:formKey'),
                id: dom.getAttribute('id'),
                name: dom.getAttribute('name'),
                type: type
            };
            /** @type {?} */
            let shape = this.bpmnElements.createElement(xmlElement);
            if (father) {
                shape.father = father;
            }
            return shape;
        }
        catch (ex) {
            console.warn(dom);
            console.warn('Tipo não possui shape associado: ' + type);
        }
    }
    /**
     * @param {?} dom
     * @param {?} type
     * @param {?} father
     * @return {?}
     */
    createSubProcess(dom, type, father) {
        /** @type {?} */
        const childs = this.htmlCollectionToArray(dom.children);
        /** @type {?} */
        const sequences = [];
        /** @type {?} */
        const xmlElement = {
            form: dom.getAttribute('camunda:formKey'),
            id: dom.getAttribute('id'),
            name: dom.getAttribute('name'),
            type: type
        };
        /** @type {?} */
        const shape = this.bpmnElements.createElement(xmlElement);
        if (father) {
            shape.father = father;
        }
        childs.forEach((child) => {
            /** @type {?} */
            const tagName = child.tagName;
            if (tagName === 'bpmn:incoming') ;
            else if (tagName === 'bpmn:outgoing') ;
            else if (tagName === 'bpmn:sequenceFlow') {
                sequences.push(child);
            }
            else if (child.tagName !== 'bpmn:multiInstanceLoopCharacteristics') {
                shape.childrens.push(this.importElement(child, shape));
            }
        });
        sequences.forEach((sequence) => {
            this.importSequenceFlow(sequence);
        });
        return shape;
    }
    /**
     * @param {?} xml
     * @return {?}
     */
    getElements(xml) {
        this.collaboration = xml.getElementsByTagName('bpmn:collaboration')[0];
        this.process = xml.getElementsByTagName('bpmn:process')[0];
        this.diagram = xml.getElementsByTagName('bpmndi:BPMNDiagram')[0];
        this.processCollaboration();
        this.processBpmnProcess();
        this.processDiagram();
        this.bpmnElements.bindElementAndSequence();
        this.bpmnElements$.next(this.bpmnElements);
    }
    /**
     * @param {?} node
     * @return {?}
     */
    getNodePosition(node) {
        return { x: parseInt(node.getAttribute('x')), y: parseInt(node.getAttribute('y')) };
    }
    /**
     * @param {?} doms
     * @return {?}
     */
    htmlCollectionToArray(doms) {
        /** @type {?} */
        const childs = [];
        for (let i = 0; i < doms.length; i++) {
            childs.push(doms[i]);
        }
        return childs;
    }
    /**
     * @param {?} dom
     * @param {?} father
     * @return {?}
     */
    importElement(dom, father) {
        /** @type {?} */
        const type = dom.tagName.split(':')[1];
        if (type === 'subProcess') {
            return this.createSubProcess(dom, type, father);
        }
        else if (type === 'laneSet' || type === 'lane') {
            return this.createLane(dom, type, father);
        }
        else {
            return this.createShape(dom, type, father);
        }
    }
    /**
     * @param {?} sequence
     * @return {?}
     */
    importSequenceFlow(sequence) {
        /** @type {?} */
        const source = sequence.getAttribute('sourceRef');
        /** @type {?} */
        const target = sequence.getAttribute('targetRef');
        /** @type {?} */
        const id = sequence.getAttribute('id');
        /** @type {?} */
        const name = sequence.getAttribute('name');
        this.bpmnElements.createSequence(id, name, source, target);
    }
    /**
     * @return {?}
     */
    processCollaboration() {
        /** @type {?} */
        const childs = this.htmlCollectionToArray(this.collaboration.children);
        for (let child of childs) {
            if (child.tagName === 'bpmn:participant') {
                this.importElement(child, null);
            }
            else {
                console.warn('Collaboration not known: ' + child.tagName);
            }
        }
    }
    /**
     * @return {?}
     */
    processBpmnProcess() {
        /** @type {?} */
        const childs = this.htmlCollectionToArray(this.process.children);
        /** @type {?} */
        const sequenceFlows = childs.filter((child) => child.tagName === 'bpmn:sequenceFlow');
        /** @type {?} */
        const elements = childs.filter((child) => child.tagName !== 'bpmn:sequenceFlow');
        this.bpmnElements.processId = this.process.getAttribute('id');
        this.bpmnElements.processName = this.process.getAttribute('name');
        elements.forEach((element) => this.importElement(element, null));
        sequenceFlows.forEach((sequence) => this.importSequenceFlow(sequence));
    }
    /**
     * @return {?}
     */
    processDiagram() {
        /** @type {?} */
        const plane = this.diagram.getElementsByTagName('bpmndi:BPMNPlane')[0];
        /** @type {?} */
        const childs = plane.childNodes;
        childs.forEach((child) => {
            if (child.tagName) {
                if (child.tagName === 'bpmndi:BPMNShape') {
                    this.processShapes(child);
                }
                else if (child.tagName === 'bpmndi:BPMNEdge') {
                    this.processSequences(child);
                }
                else {
                    console.warn('Elemento do diagrama não conhecido: ' + child.tagName);
                }
            }
        });
    }
    /**
     * @param {?} child
     * @return {?}
     */
    processSequences(child) {
        /** @type {?} */
        const id = child.getAttribute('bpmnElement');
        /** @type {?} */
        const childNodes = child.childNodes;
        childNodes.forEach((node) => {
            this.processSequenceChild(id, node);
        });
    }
    /**
     * @param {?} id
     * @param {?} node
     * @return {?}
     */
    processSequenceChild(id, node) {
        if (node.nodeName !== '#text') {
            if (node.tagName === 'di:waypoint') {
                /** @type {?} */
                let position = this.getNodePosition(node);
                this.bpmnElements.setSequenceWaypoints(id, position);
            }
            else if (node.tagName === 'bpmndi:BPMNLabel') {
                /** @type {?} */
                let position = this.getNodePosition(node.childNodes[1]);
                this.bpmnElements.setSequenceNamePlacement(id, position);
            }
        }
    }
    /**
     * @param {?} child
     * @return {?}
     */
    processShapes(child) {
        /** @type {?} */
        const id = child.getAttribute('bpmnElement');
        /** @type {?} */
        const childNodes = child.childNodes;
        childNodes.forEach((node) => {
            this.processShapeChild(id, node);
        });
    }
    /**
     * @param {?} id
     * @param {?} node
     * @return {?}
     */
    processShapeChild(id, node) {
        if (node.nodeName !== '#text') {
            if (node.tagName === 'dc:Bounds') {
                /** @type {?} */
                let position = this.getNodePosition(node);
                /** @type {?} */
                let height = parseInt(node.getAttribute('height'));
                /** @type {?} */
                let width = parseInt(node.getAttribute('width'));
                this.bpmnElements.setShapeAttributes(id, width, height, position);
            }
            else if (node.tagName === 'bpmndi:BPMNLabel') {
                /** @type {?} */
                let position = this.getNodePosition(node.childNodes[1]);
                this.bpmnElements.setShapeNamePlacement(id, position);
            }
            else {
                console.warn(node);
                console.warn('Elemento não conhecido.');
            }
        }
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class Scene {
    /**
     * @param {?} bpmnElements
     * @param {?} sceneClass
     */
    constructor(bpmnElements, sceneClass) {
        this.bpmnElements = bpmnElements;
        this.sceneClass = sceneClass;
        this.centerEye = {};
        this.loaded$ = new BehaviorSubject(false);
        this.svgScene = null;
        this.svgSceneElements = null;
        this.svgSceneShapes = null;
        this.svgSceneSequences = null;
        this.svgSceneDefs = null;
        this.sequences = this.bpmnElements.sequences;
        this.shapes = this.bpmnElements.shapes;
    }
    /**
     * @return {?}
     */
    init() {
        this.loadSceneEngine();
        this.loadElements();
        this.calculateCenterEye();
        this.loaded$.next(true);
    }
    /**
     * @return {?}
     */
    calculateCenterEye() {
        /** @type {?} */
        let bpmnFlowsContainer = document.getElementsByClassName(this.sceneClass)[0];
        /** @type {?} */
        let bpmnWidth = Math.abs(this.centerEye.maxX - this.centerEye.minX);
        /** @type {?} */
        let bpmnHeight = Math.abs(this.centerEye.maxY - this.centerEye.minY);
        /** @type {?} */
        let translateXFix = (this.centerEye.minX < 0) ? this.centerEye.minX * -1 : 0;
        /** @type {?} */
        let translateYFix = (this.centerEye.minY < 0) ? this.centerEye.minY * -1 : 0;
        /** @type {?} */
        let width = bpmnFlowsContainer.clientWidth;
        /** @type {?} */
        let height = bpmnFlowsContainer.clientHeight;
        /** @type {?} */
        let scale = 1;
        while ((bpmnWidth * scale) > width || (bpmnHeight * scale) > height) {
            scale -= 0.03;
        }
        /** @type {?} */
        let translateX = translateXFix + 20;
        /** @type {?} */
        let translateY = translateYFix;
        /** @type {?} */
        let centeredZoom = zoomIdentity.translate(translateX, translateY).scale(scale);
        this.zoomEye.call(this.zoomBehaviour.transform, centeredZoom);
    }
    /**
     * @param {?} sequence
     * @return {?}
     */
    calculatePath(sequence) {
        /** @type {?} */
        const waypoints = sequence.element.waypoints;
        /** @type {?} */
        let d = 'm ' + waypoints[0].x + ', ' + waypoints[0].y;
        for (let waypoint = 1; waypoint < waypoints.length; waypoint++) {
            d += 'L' + waypoints[waypoint].x + ',' + waypoints[waypoint].y;
        }
        return d;
    }
    /**
     * @param {?} element
     * @return {?}
     */
    calculateTextposition(element) {
        /** @type {?} */
        let x = element.textPosition.x - element.position.x + 20;
        /** @type {?} */
        let y = element.textPosition.y - element.position.y + 10;
        return x + ',' + y;
    }
    /**
     * @return {?}
     */
    loadDefs() {
        this.svgSceneDefs.append('defs').append('marker')
            .attr('id', 'marker_arrow')
            .attr('markerHeight', 10)
            .attr('markerWidth', 12)
            .attr('markerUnits', 'strokeWidth')
            .attr('orient', 'auto')
            .attr('refX', 9.5)
            .attr('refY', 10)
            .attr('viewBox', '0 0 22 22')
            .append('path')
            .attr('d', 'M 1 5 L 11 10 L 1 15 Z')
            .attr('class', 'bpmn-flows-marker');
    }
    /**
     * @return {?}
     */
    loadElements() {
        this.loadDefs();
        this.loadShapes();
        this.loadSequences();
    }
    /**
     * @return {?}
     */
    loadShapes() {
        /** @type {?} */
        const shapesKeys = Object.keys(this.shapes);
        for (let shapesKey of shapesKeys) {
            /** @type {?} */
            let shape = this.shapes[shapesKey];
            shape.element.svgElement = this.svgSceneShapes.append('g');
            if (shape.element instanceof PrimitiveCircle) {
                this.renderCircle(shape, shape.element);
            }
            else if (shape.element instanceof PrimitiveRect) {
                this.renderRect(shape, shape.element);
            }
            else if (shape.element instanceof PrimitiveRhombus) {
                this.renderRhombus(shape, shape.element);
            }
        }
    }
    /**
     * @return {?}
     */
    loadSceneEngine() {
        this.svgScene = select('.' + this.sceneClass).append('svg');
        this.svgScene.attr('class', 'bpmn-flows-svg-scene');
        this.svgSceneElements = this.svgScene.append('g');
        this.svgSceneDefs = this.svgSceneElements.append('g');
        this.svgSceneSequences = this.svgSceneElements.append('g');
        this.svgSceneShapes = this.svgSceneElements.append('g');
        this.zoomEngine();
    }
    /**
     * @return {?}
     */
    loadSequences() {
        /** @type {?} */
        const sequencesKeys = Object.keys(this.sequences);
        for (let sequenceKey of sequencesKeys) {
            /** @type {?} */
            let sequence = this.sequences[sequenceKey];
            sequence.element.svgElement = this.svgSceneSequences.append('g');
            this.renderSequence(sequence);
        }
    }
    /**
     * @param {?} shape
     * @param {?} element
     * @return {?}
     */
    renderCircle(shape, element) {
        /** @type {?} */
        let name = shape.name;
        /** @type {?} */
        let circle = element.svgElement.append('circle');
        circle.attr('cx', '16');
        circle.attr('cy', '16');
        circle.attr('r', element.ratio);
        circle.attr('class', element.cssClass);
        if (name) {
            /** @type {?} */
            let text = element.svgElement.append('text');
            /** @type {?} */
            let translate = this.calculateTextposition(element);
            text.attr('class', element.textCssClass);
            text.attr('transform', 'translate(' + translate + ')');
            text.text(name);
        }
        element.svgElement.attr('transform', 'translate( ' + element.position.x + ', ' + element.position.y + ' )');
        this.setCenterEye(element.position, element.width, element.height);
    }
    /**
     * @param {?} shape
     * @param {?} element
     * @return {?}
     */
    renderRect(shape, element) {
        /** @type {?} */
        let rec = element.svgElement.append('rect');
        rec.attr('id', shape.id);
        rec.attr('width', element.width);
        rec.attr('height', element.height);
        rec.attr('class', element.cssClass);
        if (element instanceof MultiInstanceRect) {
            this.renderMultiInstance(shape, element);
        }
        else if (element instanceof ParticipantRect || element instanceof LaneRect) {
            this.renderLane(shape, element);
        }
        else {
            this.renderTask(shape, element);
        }
        element.svgElement.attr('transform', 'translate( ' + element.position.x + ', ' + element.position.y + ' )');
        this.setCenterEye(element.position, element.width, element.height);
    }
    /**
     * @param {?} shape
     * @param {?} element
     * @return {?}
     */
    renderRhombus(shape, element) {
        /** @type {?} */
        let polygon = element.svgElement.append('polygon');
        /** @type {?} */
        let foreignObject = element.svgElement.append('foreignObject').attr('y', 0).attr('x', 0);
        /** @type {?} */
        let div = foreignObject.append('xhtml:div');
        /** @type {?} */
        let name = shape.name;
        polygon.attr('points', element.points);
        polygon.attr('width', element.width);
        polygon.attr('height', element.height);
        polygon.attr('class', element.cssClass);
        if (name) {
            /** @type {?} */
            let text = element.svgElement.append('text');
            /** @type {?} */
            let translate = this.calculateTextposition(element);
            text.attr('class', element.textCssClass);
            text.attr('transform', 'translate(' + translate + ')');
            text.text(name);
        }
        div.attr('style', 'float:left; width: ' + (element.width) + 'px; height:' + (element.height) + 'px;');
        div.attr('class', element.icon + ' ' + element.iconCssClass);
        element.svgElement.attr('transform', 'translate( ' + element.position.x + ', ' + element.position.y + ' )');
        this.setCenterEye(element.position, element.width, element.height);
    }
    /**
     * @param {?} sequence
     * @return {?}
     */
    renderSequence(sequence) {
        /** @type {?} */
        let sequenceElement = /** @type {?} */ (sequence.element);
        /** @type {?} */
        let path = sequenceElement.svgElement.append('path');
        path.attr('id', sequence.id);
        path.attr('class', sequenceElement.cssClass);
        path.attr('d', this.calculatePath(sequence));
        path.attr('marker-end', 'url(#marker_arrow)');
        if (sequence.name) {
            /** @type {?} */
            let text = sequenceElement.svgElement.append('text');
            text.attr('class', sequenceElement.textCssClass);
            text.attr('transform', 'translate(' + (sequenceElement.textPosition.x) + ',' + (sequenceElement.textPosition.y + 10) + ')');
            text.text(sequence.name);
        }
    }
    /**
     * @param {?} shape
     * @param {?} element
     * @return {?}
     */
    renderLane(shape, element) {
        /** @type {?} */
        let text = element.svgElement.append('text');
        text.attr('class', element.textCssClass);
        text.attr('x', -element.height / 2);
        text.attr('y', 15);
        text.text(shape.name);
    }
    /**
     * @param {?} shape
     * @param {?} element
     * @return {?}
     */
    renderMultiInstance(shape, element) {
        /** @type {?} */
        let name = shape.name;
        if (name) {
            /** @type {?} */
            let text = element.svgElement.append('text');
            text.attr('class', element.textCssClass);
            text.text(name);
            text.attr('line-height', '1.2');
            text.attr('y', 20);
            text.attr('x', element.width / 2);
        }
    }
    /**
     * @param {?} shape
     * @param {?} element
     * @return {?}
     */
    renderTask(shape, element) {
        /** @type {?} */
        let taskName = shape.name;
        if (element.icon) {
            element.svgElement.append('foreignObject').attr('y', 5).attr('x', 5)
                .append('xhtml:span').attr('class', element.iconCssClass + ' ' + element.icon);
        }
        if (taskName) {
            /** @type {?} */
            let text = element.svgElement.append('text');
            text.attr('class', element.textCssClass);
            text.text(taskName);
            text.attr('line-height', '1.2');
            text.attr('y', (element.height / 2) + 5);
            text.attr('x', element.width / 2);
        }
        this.setCenterEye(element.position, element.width, element.height);
    }
    /**
     * @param {?} position
     * @param {?} width
     * @param {?} height
     * @return {?}
     */
    setCenterEye(position, width, height) {
        if (!this.centerEye.maxX) {
            this.centerEye.maxX = position.x;
            this.centerEye.minX = position.x;
            this.centerEye.minY = position.y;
            this.centerEye.maxY = position.y;
            return;
        }
        if (this.centerEye.minX > position.x) {
            this.centerEye.minX = position.x;
        }
        if (this.centerEye.maxX < position.x + width) {
            this.centerEye.maxX = position.x + width;
        }
        if (this.centerEye.minY > position.y) {
            this.centerEye.minY = position.y;
        }
        if (this.centerEye.maxY < position.y + height) {
            this.centerEye.maxY = position.y + height;
        }
    }
    /**
     * @param {?} transform
     * @param {?} shapes
     * @return {?}
     */
    zoomed(transform, shapes) {
        shapes.attr('transform', transform);
    }
    /**
     * @return {?}
     */
    zoomEngine() {
        this.zoomBehaviour = zoom()
            .scaleExtent([1 / 2, 4])
            .on('zoom', () => this.zoomed(event.transform, this.svgSceneElements));
        this.zoomEye = this.svgScene.append('rect');
        this.zoomEye.attr('class', 'bpmn-flows-zoom-eye');
        this.zoomEye.call(this.zoomBehaviour);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class BpmnFlowsService {
    /**
     * @param {?} http
     */
    constructor(http) {
        this.http = http;
    }
    /**
     * @param {?} bpmnFile
     * @return {?}
     */
    initialize(bpmnFile) {
        this.getBpmnDiagram(bpmnFile);
    }
    /**
     * @param {?} text
     * @return {?}
     */
    bpmnDiagramParser(text) {
        /** @type {?} */
        const parser = new DOMParser();
        /** @type {?} */
        const xml = parser.parseFromString(text, "text/xml");
        this.bpmnImporter = new BpmnImporter(xml);
        this.eventImportedBpmn();
    }
    /**
     * @return {?}
     */
    eventImportedBpmn() {
        this.bpmnImporter.bpmnElements$.subscribe((bpmnElements) => {
            this.bpmnElements = bpmnElements;
            if (this.bpmnElements) {
                this.renderBpmn();
            }
        });
    }
    /**
     * @return {?}
     */
    eventRenderedBpmn() {
        this.scene.loaded$.subscribe((loaded) => {
        });
    }
    /**
     * @param {?} bpmnFilePath
     * @return {?}
     */
    getBpmnDiagram(bpmnFilePath) {
        this.http.get(bpmnFilePath, { responseType: 'text' }).subscribe((response) => {
            this.bpmnDiagramParser(response);
        });
    }
    /**
     * @return {?}
     */
    renderBpmn() {
        this.scene = new Scene(this.bpmnElements, 'bpmn-flows-container');
        this.scene.init();
        this.eventRenderedBpmn();
    }
}
BpmnFlowsService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
BpmnFlowsService.ctorParameters = () => [
    { type: HttpClient }
];
/** @nocollapse */ BpmnFlowsService.ngInjectableDef = defineInjectable({ factory: function BpmnFlowsService_Factory() { return new BpmnFlowsService(inject(HttpClient)); }, token: BpmnFlowsService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class BpmnFlowsComponent {
    /**
     * @param {?} bpmnFlowsService
     */
    constructor(bpmnFlowsService) {
        this.bpmnFlowsService = bpmnFlowsService;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.bpmnFlowsService.initialize(this.fileUrl);
    }
}
BpmnFlowsComponent.decorators = [
    { type: Component, args: [{
                selector: 'bpmn-flows',
                template: "<div class = \"bpmn-flows-container\">\r\n</div>",
                styles: [".bpmn-flows-container{float:left;box-sizing:border-box;height:100%;width:100%;position:relative;overflow-x:hidden;overflow-y:hidden}.bpmn-flows-svg-scene{height:5000px;width:5000px;position:relative}.bpmn-flows-circle,.bpmn-flows-gateway,.bpmn-flows-rect{fill:none;stroke:#6b6a6a}.bpmn-flows-end-event-circle{stroke-width:3px}.bpmn-flows-zoom-eye{fill:none;pointer-events:all;width:100%;height:100%}.bpmn-flows-marker{fill:#6b6a6a}.bpmn-flows-path{fill:none;stroke-width:2px;stroke:#6b6a6a;opacity:.8}.bpmn-flows-gateway-icon,.bpmn-flows-task-icon{float:left;color:#6b6a6a;font-size:14px}.bpmn-flows-gateway-icon{float:left;font-size:18px;text-align:center;padding-top:15px}.bpmn-flows-gateway-icon span{margin:auto}.bpmn-flows-shadow{box-shadow:inset 0 0 1px 0 #6b6a6a}.bpmn-flows-circle-name,.bpmn-flows-lane-name,.bpmn-flows-path-name,.bpmn-flows-text{fill:#6b6a6a;text-anchor:middle;font-family:Verdana,Helvetica;font-size:12px;text-align:center}.bpmn-flows-lane-name{-webkit-transform:rotate(-90deg);transform:rotate(-90deg)}.bpmn-flows-icon{font-family:'Font Awesome 5 Free'}"]
            }] }
];
/** @nocollapse */
BpmnFlowsComponent.ctorParameters = () => [
    { type: BpmnFlowsService }
];
BpmnFlowsComponent.propDecorators = {
    fileUrl: [{ type: Input, args: ['fileUrl',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class BpmnFlowsModule {
}
BpmnFlowsModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    HttpClientModule
                ],
                providers: [HttpClient],
                declarations: [BpmnFlowsComponent],
                exports: [BpmnFlowsComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

export { BpmnFlowsService, BpmnFlowsComponent, BpmnFlowsModule };

//# sourceMappingURL=bpmn-flows.js.map